#

#    Task to obtain various rate information and to monitor the system

#    1/10/97

#    monitors state of tape drives and can provide warning and/or halt DA
#    when approaching end of tape

set debug 0
set started 0

set frame RATES
set frametitle "Rates and Monitoring"
set icontitle Rates

set taperateperiod 15
set tapemonitorperiod 100

set taperatesstarted 0

set tapemonitorstarted 0

set autorestartenable 0
set autorestartcount 0

#    Tape drive rates
#    just handle the first 4 drives

set tapestate 0
set taperates {0 0 0 0}
set tapeusedpercent {0 0 0 0}

#if {[exec hostname] == "eb63"} then {
    set tapewarn 90
    set tapehalt 95
#} else {
#    set tapewarn disabled
#    set tapehalt disabled
#}

set EGreport -1
set EGallow 0xffffffff


proc make-frame {} {
  global env
  global taperates tapeusedpercent tapewarn tapehalt

  set warnchoice [list 80 85 90 92 94 96 98 disabled]
  set haltchoice [list 90 92 94 95 96 98 99 disabled]

  :frame "[self frame-name]" :label "[self frame-title]" :show-footer T  :resize F \
      :icon-image "@$env(ICONHOME)/default" \
      :icon-name "[self icon-title]"

    :panel PANEL1 :layout T :padx 10 :pady 10
      :layout H; :row-gap 5; :col-gap 10
      :message M1    :label "Rate Monitoring" :bold T :valuex 25c
      :next-row T
      :message TAPERATES :label "Tape data rates (Kbytes/sec) -  $taperates"
      :next-row T
      :message M2    :label "Tape Drive Monitoring" :bold T :valuex 25c
      :next-row T
      :message TAPEUSED :label "Tape used (%) -  $tapeusedpercent"
      :next-row T
      :choice-stack TAPEWARN   :label "Warning when tape used is (%)" \
           :strings $warnchoice :value [lsearch $warnchoice $tapewarn]
      :choice-stack TAPEHALT   :label "Halt when tape used is (%)" \
           :strings $haltchoice :value [lsearch $haltchoice $tapehalt]

    :panel BUTTONS :layout B :padx 10 :pady 10 :after FOOTER
    :layout H; :col-gap 10; :row-gap 10
      :button STARTM       :label "Start tape rate monitoring"
      :button STARTTAPER   :label "Start tape drive monitoring"
   :show T
}


proc click arg {
  global tapewarn tapehalt taperatesstarted tapemonitorstarted
   set f [lindex $arg 0]
   set c [lindex $arg 1]
   set w [lindex $arg 2]
   set v [lindex $arg 3]
   set i [lindex $arg 4]
   if {$c==":quit"} then {return [click-quit]}
   switch $w  {
      TAPEWARN  {set tapewarn $i}
      TAPEHALT  {set tapehalt $i}
      STARTTAPER    {
         if {$taperatesstarted == 0} then {start-tape-rates} else {stop-tape-rates}
      }
      STARTM    {
         if {$tapemonitorstarted == 0} then {start-tapemonitor} else {stop-tapemonitor}
      }
      default {click-[string tolower $w] $v}
   }
}

proc redisplay {} {
  global taperates tapeusedpercent taperatesstarted tapemonitorstarted

  :modify TAPERATES :label "Tape data rates (Kbytes/sec) -  $taperates"
  :modify TAPEUSED  :label "Tape used (%) -  $tapeusedpercent"
  if {$taperatesstarted == 0} then {
     :modify STARTTAPER  :label "Start tape rate monitoring"
  } else {
     :modify STARTTAPER  :label "Stop tape rate monitoring"
  }
  if {$tapemonitorstarted == 0} then {
     :modify STARTM  :label "Start tape drive monitoring"
  } else {
     :modify STARTM  :label "Stop tape drive monitoring"
  }
  set-footer "Last Updated [date]"
}


proc update-tape-rates {} {
  global tapestate taperates
   set tapestate [inform tape read-egstate]
   if {$tapestate < 1} then {
#    STOPped - set rates to zero
       set taperates {0 0 0 0}
   } else {
#    Going
       inform tape update-information
       set taperates [inform tape inquire taperates]
   }
#  insert-debug "$taperates"
}

proc tape-warning {} {
  global tapewarn tapehalt tapestate tapeusedpercent
   set tapestate [inform tape read-egstate]
   set tapeusedpercent {}
   if {$tapestate < 1} then {return}
   set tapeusedpercent [inform tape inquire tapepercent]
   for {set i 0} {$i < [llength $tapeusedpercent]} {incr i} {
      set percent [string range [lindex $tapeusedpercent $i] 0 3]
      if {$tapewarn != "disabled" && $percent >= $tapewarn} then {
#    generate warning
         warning-msg "You are near the end of tape - $percent% used"
         bell
      }
      if {$tapehalt != "disabled" && $percent >= $tapehalt} then {
#    halt DA
         warning-msg "You are too near the end of tape ($percent% used)\n****Data Acquisition STOPped****"
         bell;bell;bell;bell;bell
         inform sys "click-da 0"
         break
      }
   }
}

proc warning-msg {msg} {

   open-frame

  :frame [self frame-name].warning :label "Tape Warning" :show-footer T :resize F :transient T

    :panel A

      :font local message b :pxlsz 24
      :message M :label "Warning --- $msg" :bold T

      :show T
}

proc start-tape-rates {} {
  global taperateperiod taperatesstarted taperates
   if {$taperatesstarted == 1} then {return}
   set taperates {0 0 0 0}
   update-tape-rates
   after [expr $taperateperiod * 1000] continue-tape-rates
   set taperatesstarted 1
   redisplay
}

proc stop-tape-rates {} {
   global taperatesstarted taperates
   set taperatesstarted 0
   set taperates {0 0 0 0}
   redisplay
}

proc start-tapemonitor {} {
  global tapemonitorperiod tapemonitorstarted
   if {$tapemonitorstarted == 1} then {return}
   after [expr $tapemonitorperiod * 1000] continue-tape-monitor
   set tapemonitorstarted 1
   tape-warning
   redisplay
}

proc stop-tapemonitor {} {
   global tapemonitorstarted tapeusedpercent
   set tapemonitorstarted 0
   set tapeusedpercent {}
   redisplay
}

proc continue-tape-rates {} {
  global taperateperiod taperatesstarted
   if {$taperatesstarted == 0} then {return}
   after [expr $taperateperiod * 1000] continue-tape-rates
   update-tape-rates
   redisplay
}

proc continue-tape-monitor {} {
   global tapemonitorperiod tapemonitorstarted
   if {$tapemonitorstarted == 0} then {return}
   after [expr $tapemonitorperiod * 1000] continue-tape-monitor
   tape-warning
   redisplay
}


