
proc Set_V1190_Options {} {
    global Options V1190_Base

    load_Options

   if {![info exists Options(v1190.channels)]} {set Options(v1190.channels) 4096}
   if {![info exists Options(v1190.offset)]} {set Options(v1190.offset) 0}
   if {![info exists Options(v1190.shift)]} {set Options(v1190.shift) 4}
   if {![info exists Options(v1190_GroupBase)]} {set Options(v1190_GroupBase) $V1190_Base}
   set V1190_Base   $Options(v1190_GroupBase)

    update_Options
}

proc Dump_Raw_VME_Data_V1190 {data} {

   set info ""
      
     foreach word $data {
         if {[string range $word 0 1] != "0x"} then {set word 0x$word}
         if {$word == 0xaa5555aa} then {
             append info "$word - end of dump\n"
             break
         }
         if {$word == 0x55aaaa55} then {
             append info "$word - end of event\n"
             continue
         }
         set type [expr $word >> 30 & 3]
         switch $type {
            0 {
               set subtype [expr $word >> 27 & 7]
               switch $subtype {
                   0 {
                     set event_type "TDC Data"
                     set channel [expr $word >> 19 & 0x7f]
                     set ed [expr $word >> 26 & 1]
                     set tdc [expr $word  & 0x7ffff]
                     append info "[format "0x%8x    %8s   channel= %5i; edge= %3i; time= 0x%5x" $word $event_type $channel $ed $tdc]"
                   }
                   1 {
                     set event_type "TDC Header"
                     set tdc [expr $word >> 24 & 0x3]
                     set event [expr $word >> 12 & 0x0fff]
                     set bunch [expr $word  & 0x0fff]
                     append info "[format "0x%8x    %16s   tdc= %3i; event ID= %8i; bunch ID= %6i" $word  $event_type $tdc $event $bunch]"
                   }
                   3 {
                     set event_type "TDC Trailer"
                     set tdc [expr $word >> 24 & 0x3]
                     set event [expr $word >> 12 & 0x0fff]
                     set wc [expr $word  & 0x0fff]
                     append info "[format "0x%8x    %16s   tdc= %3i; event ID= %8i; word count= %6i" $word  $event_type $tdc $event $wc]"
                   }
                   4 {
                     set event_type "TDC Error"
                     set tdc [expr $word >> 24 & 0x3]
                     set flags [expr $word  & 0x3fff]
                     append info "[format "0x%8x    %16s   tdc= %3i; flags= 0x%5x" $word  $event_type $tdc $flags]"
                   }
                   default {append info "[format "0x%8x    %16s" $word  "Unknown"]"}
               }
            }
            1 {
             set event_type "Global Header"
             set geo [expr $word & 0x1f]
             set counter [expr $word >> 5 & 0x3fffff]
             append info "[format "0x%8x    %8s   module= %5i; event number= %8i" $word $event_type $geo $counter]"
            }
            2 {
             set event_type "Global Trailer"
             set geo [expr $word & 0x1f]
             set status [expr $word >> 24 & 7]
             set counter [expr $word >> 5 & 0xffff]
             append info "[format "0x%8x    %16s   module= %5i; status=%2i; event data counter= %4i" $word $event_type $geo $status $counter]"
            }
            3 {
             append info "$word - fifo empty"
            }
         }
         append info "\n"
     }
   return $info
}

proc define_V1190_config {} {
   global VMEserver
   global V1190_Base

   set maxmodule [inform vmecontrol inquire v1190count]
   if {[string length $maxmodule] > 2} {set maxmodule 0}
   if {$maxmodule > 0} then {DefineConfig "1190+0+[format "0x%x" $V1190_Base]"}

   for {set i 1} {$i <= $maxmodule} {incr i} {

      set z [catch {eg "(register :name '/$VMEserver/v1190#$i.Output_Buffer'  ?object)" } m]
      if {$z!=0} then {
          display-eg-error-report self $m "Attempting to locate register v1190#$i.Output_Buffer"
          clear-footer
          error $m
      }
      DefineConfig "1190+$i+[string range [first $m] 2 end]"
   }
}

proc create_V1190_spectra {} {
    global SpectrumServer VMEserver V1190_Base
    global Options

   set maxmodule [inform vmecontrol inquire v1190count]
   if {[string length $maxmodule] > 2} {set maxmodule 0}
   set maxadc 128

   for {set i 1} {$i <= $maxmodule} {incr i} {

      set-footer "creating spectra for module v1190#$i"

      set z [catch {eg "(register :name '/$VMEserver/v1190#$i.Version'  ?value)" } m]
      if {$z!=0} then {
          display-eg-error-report self $m "Attempting to read register v1190#$i.Version"
          clear-footer
          error $m
      }

      if {[expr $m & 255] == 0} {set maxadc 128} else {set maxadc 64}  ;# check for 1190A or 1190B

      set-footer "creating $maxadc spectra for module v1190#$i"

    for {set j 0} {$j < $maxadc} {incr j} {

      set spectrumname [SpectrumName v1190#$i.tdc$j]
      set z [catch {eg "(spectrum :path '/$SpectrumServer' :range $Options(v1190.channels) :name '$spectrumname' !new)" } m]

#    Allow "Spectrum already exists" so that further spectra can be created
      if {$z!=0 && $m==0x30004} then {set z 0}
      if {$z!=0} then {
          display-eg-error-report self $m "Attempting to create spectrum v1190#$i.tdc$j"
          clear-footer
          error $m
      } else {
         if {$j < 64} then {
             set group [expr $V1190_Base + (($i - 1) * 2)]
             set item $j
         } else {
             set group [expr $V1190_Base + (($i - 1) * 2) + 1]
             set item [expr $j - 64]
         }
          map_spectrum  "$spectrumname $group $item $Options(v1190.offset) $Options(v1190.shift)"
      }
    }
   }
}

proc do_V1190_setup {action} {
    global Options

#  action = 1  - GO;   = 3  -  SETUP

   if {$action != 1 && $action != 3} {return}

   set maxmodule [inform vmecontrol inquire v1190count]
   if {[string length $maxmodule] > 2} {set maxmodule 0}

   if {$maxmodule == 0} {return}
   
   starttask v1190

   if {$action == 3} {inform v1190 Restore_Parameters; return}    ;#    restore microcontroller parameters

#   GO -  set essential options

   for {set i 1} {$i <= $maxmodule} {incr i} {

       inform v1190 click-select-module $i
               
#    acquisition mode:
#      set trigger matching
       inform v1190 do_AcqMode_opcode OPT00

#    readout 
#    disable TDC header & EOB
        inform v1190 do_Readout_opcode OPT31 0
   }
}
